# copyright 2003-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of CubicWeb.
#
# CubicWeb is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# CubicWeb is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with CubicWeb.  If not, see <https://www.gnu.org/licenses/>.
"""server.serverconfig definition"""

import sys
from io import StringIO
from os.path import join, exists

import logilab.common.configuration as lgconfig
from logilab.common.decorators import cached

from cubicweb.cwconfig import CubicWebConfiguration, CONFIG_HELP_MESSAGE
from cubicweb.server import SOURCE_TYPES
from cubicweb.toolsutils import read_config, restrict_perms_to_user

USER_OPTIONS = (
    (
        "login",
        {
            "type": "string",
            "default": "admin",
            "help": "cubicweb manager account's login " "(this user will be created)",
            "level": 0,
        },
    ),
    (
        "password",
        {
            "type": "password",
            "default": lgconfig.REQUIRED,
            "help": "cubicweb manager account's password",
            "level": 0,
        },
    ),
)


class SourceConfiguration(lgconfig.Configuration):
    def __init__(self, appconfig, options):
        self.appconfig = appconfig  # has to be done before super call
        super().__init__(options=options)

    # make Method('default_instance_id') usable in db option defs (in native.py)
    def default_instance_id(self):
        return self.appconfig.appid

    def input_option(self, option, optdict, inputlevel):
        try:
            dbdriver = self["db-driver"]
        except lgconfig.OptionError:
            pass
        else:
            if dbdriver == "sqlite":
                if option in ("db-user", "db-password"):
                    return
                if option == "db-name":
                    optdict = optdict.copy()
                    optdict["help"] = "path to the sqlite database"
                    optdict["default"] = join(
                        self.appconfig.appdatahome, self.appconfig.appid + ".sqlite"
                    )
        super().input_option(option, optdict, inputlevel)


def ask_source_config(appconfig, type, inputlevel=0):
    options = SOURCE_TYPES[type].options
    sconfig = SourceConfiguration(appconfig, options=options)
    sconfig.input_config(inputlevel=inputlevel)
    return sconfig


def generate_source_config(sconfig, encoding=sys.stdin.encoding):
    """serialize a repository source configuration as text"""
    stream = StringIO()
    optsbysect = list(sconfig.options_by_section())
    assert (
        len(optsbysect) == 1
    ), "all options for a source should be in the same group, got %s" % [
        x[0] for x in optsbysect
    ]
    lgconfig.ini_format(stream, optsbysect[0][1], encoding)
    return stream.getvalue()


class ServerConfiguration(CubicWebConfiguration):
    """standalone RQL server"""

    name = "repository"

    cubicweb_appobject_path = CubicWebConfiguration.cubicweb_appobject_path | {
        "sobjects",
        "hooks",
    }
    cube_appobject_path = CubicWebConfiguration.cube_appobject_path | {
        "sobjects",
        "hooks",
    }

    options = lgconfig.merge_options(
        (
            # ctl configuration
            (
                "host",
                {
                    "type": "string",
                    "default": None,
                    "help": "host name if not correctly detectable through gethostname",
                    "group": "main",
                    "level": 1,
                },
            ),
            (
                "pid-file",
                {
                    "type": "string",
                    "default": lgconfig.Method("default_pid_file"),
                    "help": "repository's pid file",
                    "group": "main",
                    "level": 2,
                },
            ),
            (
                "uid",
                {
                    "type": "string",
                    "default": None,
                    "help": "unix user, if this option is set, use the specified user to start \
the repository rather than the user running the command",
                    "group": "main",
                    "level": (CubicWebConfiguration.mode == "installed") and 0 or 1,
                },
            ),
            (
                "cleanup-session-time",
                {
                    "type": "time",
                    "default": "24h",
                    "help": "duration of inactivity after which a session "
                    "will be closed, to limit memory consumption (avoid sessions that "
                    "never expire and cause memory leak when http-session-time is 0, or "
                    "because of bad client that never closes their connection). "
                    "So notice that even if http-session-time is 0 and the user don't "
                    "close his browser, he will have to reauthenticate after this time "
                    "of inactivity. Default to 24h.",
                    "group": "main",
                    "level": 3,
                },
            ),
            (
                "connections-pooler-enabled",
                {
                    "type": "yn",
                    "default": True,
                    "help": "Enable the connection pooler. Set to no if you use an external database pooler (e.g. pgbouncer)",
                    "group": "main",
                    "level": 3,
                },
            ),
            (
                "connections-pool-max-size",
                {
                    "type": "int",
                    "default": 0,
                    "help": "Maximum, per process, number of database connections. Default 0 (unlimited)",
                    "group": "main",
                    "level": 3,
                },
            ),
            (
                "connections-pool-min-size",
                {
                    "type": "int",
                    "default": 0,
                    "help": "Minimum, per process, number of database connections.",
                    "group": "main",
                    "level": 3,
                },
            ),
            (
                "connections-pool-idle-timeout",
                {
                    "type": "int",
                    "default": 600,
                    "help": "Start closing connection if the pool hasn't been empty for this many seconds",
                    "group": "main",
                    "level": 3,
                },
            ),
            (
                "rql-cache-size",
                {
                    "type": "int",
                    "default": 3000,
                    "help": "size of the parsed rql cache size.",
                    "group": "main",
                    "level": 3,
                },
            ),
            (
                "undo-enabled",
                {
                    "type": "yn",
                    "default": False,
                    "help": "enable undo support",
                    "group": "main",
                    "level": 3,
                },
            ),
            (
                "keep-transaction-lifetime",
                {
                    "type": "int",
                    "default": 7,
                    "help": "number of days during which transaction records should be \
kept (hence undoable).",
                    "group": "main",
                    "level": 3,
                },
            ),
            (
                "multi-sources-etypes",
                {
                    "type": "csv",
                    "default": (),
                    "help": "defines which entity types from this repository are used \
by some other instances. You should set this properly for these instances to \
detect updates / deletions.",
                    "group": "main",
                    "level": 3,
                },
            ),
            (
                "delay-full-text-indexation",
                {
                    "type": "yn",
                    "default": False,
                    "help": "When full text indexation of entity has a too important cost"
                    " to be done when entity are added/modified by users, activate this "
                    "option and setup a job using cubicweb-ctl db-rebuild-fti on your "
                    "system (using cron for instance).",
                    "group": "main",
                    "level": 3,
                },
            ),
            # email configuration
            (
                "default-recipients-mode",
                {
                    "type": "choice",
                    "choices": ("default-dest-addrs", "users", "none"),
                    "default": "default-dest-addrs",
                    "help": 'when a notification should be sent with no specific rules \
to find recipients, recipients will be found according to this mode. Available \
modes are "default-dest-addrs" (emails specified in the configuration \
variable with the same name), "users" (every users which has activated \
account with an email set), "none" (no notification).',
                    "group": "email",
                    "level": 2,
                },
            ),
            (
                "default-dest-addrs",
                {
                    "type": "csv",
                    "default": (),
                    "help": "comma separated list of email addresses that will be used \
as default recipient when an email is sent and the notification has no \
specific recipient rules.",
                    "group": "email",
                    "level": 2,
                },
            ),
            (
                "supervising-addrs",
                {
                    "type": "csv",
                    "default": (),
                    "help": "comma separated list of email addresses that will be \
notified of every changes.",
                    "group": "email",
                    "level": 2,
                },
            ),
        )
        + CubicWebConfiguration.options
    )

    # read the schema from the database
    read_instance_schema = True
    # set this to true to get a minimal repository, for instance to get cubes
    # information on commands such as i18ninstance, db-restore, etc...
    quick_start = False
    # check user's state at login time
    consider_user_state = True

    # should some hooks be deactivated during [pre|post]create script execution
    free_wheel = False

    # list of enables sources when sources restriction is necessary
    # (eg repository initialization at least)
    enabled_sources = None

    def bootstrap_cubes(self):
        from logilab.common.textutils import splitstrip

        with open(join(self.apphome, "bootstrap_cubes")) as f:
            for line in f:
                line = line.strip()
                if not line or line.startswith("#"):
                    continue
                self.init_cubes(self.expand_cubes(splitstrip(line)))
                break
            else:
                # no cubes
                self.init_cubes(())

    def write_bootstrap_cubes_file(self, cubes):
        stream = open(join(self.apphome, "bootstrap_cubes"), "w")
        stream.write("# this is a generated file only used for bootstraping\n")
        stream.write("# you should not have to edit this\n")
        stream.write("%s\n" % ",".join(cubes))
        stream.close()

    def sources_file(self):
        return join(self.apphome, "sources")

    # this method has to be cached since when the server is running using a
    # restricted user, this user usually don't have access to the sources
    # configuration file (#16102)
    @cached
    def read_sources_file(self):
        """return a dictionary of values found in the sources file"""
        return read_config(self.sources_file(), raise_if_unreadable=True)

    @property
    def system_source_config(self):
        return self.read_sources_file()["system"]

    @property
    def default_admin_config(self):
        return self.read_sources_file()["admin"]

    def source_enabled(self, source):
        if self.sources_mode is not None:
            if "migration" in self.sources_mode:
                assert len(self.sources_mode) == 1
                if source.connect_for_migration:
                    return True
                print("not connecting to source", source.uri, "during migration")
                return False
            if "all" in self.sources_mode:
                assert len(self.sources_mode) == 1
                return True
            return source.uri in self.sources_mode
        if self.quick_start:
            return source.uri == "system"
        return not source.disabled and (
            not self.enabled_sources or source.uri in self.enabled_sources
        )

    def write_sources_file(self, sourcescfg):
        """serialize repository'sources configuration into a INI like file"""
        sourcesfile = self.sources_file()
        if exists(sourcesfile):
            import shutil

            shutil.copy(sourcesfile, sourcesfile + ".bak")
        stream = open(sourcesfile, "w")
        stream.write(CONFIG_HELP_MESSAGE)
        for section in ("admin", "system"):
            sconfig = sourcescfg[section]
            if isinstance(sconfig, dict):
                # get a Configuration object
                assert section == "system", "%r is not system" % section
                _sconfig = SourceConfiguration(
                    self, options=SOURCE_TYPES["native"].options
                )
                for attr, val in sconfig.items():
                    try:
                        _sconfig.set_option(attr, val)
                    except lgconfig.OptionError:
                        # skip adapter, may be present on pre 3.10 instances
                        if attr != "adapter":
                            self.error("skip unknown option %s in sources file" % attr)
                sconfig = _sconfig
            stream.write("[{}]\n{}\n".format(section, generate_source_config(sconfig)))
        restrict_perms_to_user(sourcesfile)

    def load_schema(self, expand_cubes=False, **kwargs):
        from cubicweb.schema import CubicWebSchemaLoader

        if expand_cubes:
            # in case some new dependencies have been introduced, we have to
            # reinitialize cubes so the full filesystem schema is read
            origcubes = self.cubes()
            self._cubes = None
            self.init_cubes(self.expand_cubes(origcubes))
        schema = CubicWebSchemaLoader().load(self, **kwargs)
        if expand_cubes:
            # restore original value
            self._cubes = origcubes
        return schema

    def load_bootstrap_schema(self):
        from cubicweb.schema import BootstrapSchemaLoader

        schema = BootstrapSchemaLoader().load(self)
        schema.name = "bootstrap"
        return schema

    sources_mode = None

    def set_sources_mode(self, sources):
        self.sources_mode = sources

    def migration_handler(
        self,
        schema=None,
        interactive=True,
        cnx=None,
        repo=None,
        connect=True,
        verbosity=None,
    ):
        """return a migration handler instance"""
        from cubicweb.server.migractions import ServerMigrationHelper

        if verbosity is None:
            verbosity = getattr(self, "verbosity", 0)
        return ServerMigrationHelper(
            self,
            schema,
            interactive=interactive,
            cnx=cnx,
            repo=repo,
            connect=connect,
            verbosity=verbosity,
        )



# copyright 2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2014-2016 UNLISH S.A.S. (Montpellier, FRANCE), all rights reserved.
#
# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of CubicWeb.
#
# CubicWeb is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# CubicWeb is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with CubicWeb.  If not, see <https://www.gnu.org/licenses/>.

"""Binding of CubicWeb connection to Pyramid request."""

import itertools
import logging
import warnings
from cgi import FieldStorage
from contextlib import contextmanager

import rql
from pyramid import httpexceptions

import cubicweb_web
from cubicweb.pyramid import tools
from cubicweb.server import session as cwsession
from cubicweb_web.request import CubicWebRequestBase

log = logging.getLogger(__name__)


class Connection(cwsession.Connection):
    """A specialised Connection that access the session data through a
    property.

    This behavior makes sure the actual session data is not loaded until
    actually accessed.
    """

    def __init__(self, session, *args, **kw):
        super().__init__(session._repo, session._user, *args, **kw)
        self.session = session
        self.lang = session._cached_lang

    def _get_session_data(self):
        return self.session.data

    def _set_session_data(self, data):
        pass

    _session_data = property(_get_session_data, _set_session_data)


class Session:
    """A Session that access the session data through a property.

    Along with :class:`Connection`, it avoid any load of the pyramid session
    data until it is actually accessed.
    """

    def __init__(self, pyramid_request, user, repo):
        self._pyramid_request = pyramid_request
        self._user = user
        self._repo = repo

    @property
    def anonymous_session(self):
        # XXX for now, anonymous_user only exists in webconfig (and testconfig).
        # It will only be present inside all-in-one instance.
        # there is plan to move it down to global config.
        if not hasattr(self._repo.config, "anonymous_user"):
            # not a web or test config, no anonymous user
            return False
        return self._user.login == self._repo.config.anonymous_user()[0]

    def get_data(self):
        if not getattr(self, "_protect_data_access", False):
            self._data_accessed = True
            return self._pyramid_request.session

    def set_data(self, data):
        if getattr(self, "_data_accessed", False):
            self._pyramid_request.session.clear()
            self._pyramid_request.session.update(data)

    data = property(get_data, set_data)

    def new_cnx(self):
        self._protect_data_access = True
        try:
            return Connection(self)
        finally:
            self._protect_data_access = False


def cw_headers(request):
    return itertools.chain(
        *[
            [(k, item) for item in v]
            for k, v in request.cw_request.headers_out.getAllRawHeaders()
        ]
    )


@contextmanager
def cw_to_pyramid(request):
    """Context manager to wrap a call to the cubicweb API.

    All CW exceptions will be transformed into their pyramid equivalent.
    When needed, some CW reponse bits may be converted too (mainly headers)"""
    try:
        yield
    except cubicweb_web.Redirect as ex:
        assert 300 <= ex.status < 400
        raise httpexceptions.status_map[ex.status](
            ex.location, headers=cw_headers(request)
        )
    except cubicweb_web.Unauthorized:
        raise httpexceptions.HTTPForbidden(
            request.cw_request._(
                "You're not authorized to access this page. "
                "If you think you should, please contact the site "
                "administrator."
            ),
            headers=cw_headers(request),
        )
    except cubicweb_web.Forbidden:
        raise httpexceptions.HTTPForbidden(
            request.cw_request._(
                "This action is forbidden. "
                "If you think it should be allowed, please contact the site "
                "administrator."
            ),
            headers=cw_headers(request),
        )
    except (rql.BadRQLQuery, cubicweb_web.RequestError):
        raise


class CubicWebPyramidRequest(CubicWebRequestBase):
    """A CubicWeb request that only wraps a pyramid request.

    :param request: A pyramid request

    """

    def __init__(self, request):
        self._request = request

        self.path = request.upath_info

        vreg = request.registry["cubicweb.registry"]

        post = request.params.mixed()
        headers_in = request.headers

        super().__init__(vreg, post, headers=headers_in)

        self.content = request.body_file_seekable

    def setup_params(self, params):
        self._uncleaned_form = {}
        for param, val in params.items():
            if param in self.no_script_form_params and val:
                val = self.no_script_form_param(param, val)
            if isinstance(val, FieldStorage) and val.file:
                val = (val.filename, val.file)
            if param == "_cwmsgid":
                self.set_message_id(val)
            else:
                self._uncleaned_form[param] = val

    @property
    def form(self):
        class_name = self.__class__.__name__
        warnings.warn(
            f"{class_name}.form is deprecated and will be remove, uses "
            f"{class_name}.get_cleaned_form using a form_validator instead"
        )
        return self._uncleaned_form

    @form.setter
    def form(self, new_form):
        class_name = self.__class__.__name__
        warnings.warn(
            f"{class_name}.form is deprecated and will be remove, you won't be able to set it "
            "explictely, better options for specific cases where modifying it is needed"
        )
        self._uncleaned_form = new_form

    def get_cleaned_form(self, form_validator):
        return form_validator.validate(self._uncleaned_form)

    def relative_path(self, includeparams=True):
        path = self._request.path_info[1:]
        if includeparams and self._request.query_string:
            return f"{path}?{self._request.query_string}"
        return path

    def instance_uri(self):
        return self._request.application_url

    def get_full_path(self):
        path = self._request.path
        if self._request.query_string:
            return f"{path}?{self._request.query_string}"
        return path

    def http_method(self):
        return self._request.method

    def _set_status_out(self, value):
        self._request.response.status_int = value

    def _get_status_out(self):
        return self._request.response.status_int

    status_out = property(_get_status_out, _set_status_out)

    @property
    def message(self):
        """Returns a '<br>' joined list of the cubicweb current message and the
        default pyramid flash queue messages.
        """
        return "\n<br>\n".join(
            self._request.session.pop_flash()
            + self._request.session.pop_flash("cubicweb")
        )

    def set_message(self, msg):
        self.reset_message()
        self._request.session.flash(msg, "cubicweb")

    def set_message_id(self, msgid):
        self.reset_message()
        self.set_message(self._request.session.pop(msgid, ""))

    def reset_message(self):
        self._request.session.pop_flash("cubicweb")


def render_view(request, vid, **kwargs):
    """Helper function to render a CubicWeb view.

    :param request: A pyramid request
    :param vid: A CubicWeb view id
    :param kwargs: Keyword arguments to select and instanciate the view
    :returns: The rendered view content
    """
    vreg = request.registry["cubicweb.registry"]
    # XXX The select() function could, know how to handle a pyramid
    # request, and feed it directly to the views that supports it.
    # On the other hand, we could refine the View concept and decide it works
    # with a cnx, and never with a WebRequest

    with cw_to_pyramid(request):
        view = vreg["views"].select(vid, request.cw_request, **kwargs)
        view.set_stream()
        view.render()
        return view._stream.getvalue()


def _cw_cnx(request):
    """Obtains a cw session from a pyramid request

    The connection will be commited or rolled-back in a request finish
    callback (this is temporary, we should make use of the transaction manager
    in a later version).

    Not meant for direct use, use ``request.cw_cnx`` instead.

    :param request: A pyramid request
    :returns type: :class:`cubicweb.server.session.Connection`
    """
    session = request.cw_session
    if session is None:
        return None

    cnx = session.new_cnx()

    def commit_state(cnx):
        return cnx.commit_state

    def cleanup(request):
        try:
            if request.exception is not None and not isinstance(
                request.exception,
                (httpexceptions.HTTPSuccessful, httpexceptions.HTTPRedirection),
            ):
                cnx.rollback()
            elif commit_state(cnx) == "uncommitable":
                cnx.rollback()
            else:
                cnx.commit()
        finally:
            cnx.__exit__(None, None, None)

    request.add_finished_callback(cleanup)
    cnx.__enter__()
    return cnx


def repo_connect(request, repo, eid):
    """A lightweight version of
    :meth:`cubicweb.server.repository.Repository.connect` that does not keep
    track of opened sessions, removing the need of closing them"""
    user, lang = tools.cached_build_user(repo, eid)
    session = Session(request, user, repo)
    session._cached_lang = lang
    tools.cnx_attach_entity(session, user)
    return session


def _cw_session(request):
    """Obtains a cw session from a pyramid request

    :param request: A pyramid request
    :returns type: :class:`cubicweb.server.session.Session`

    Not meant for direct use, use ``request.cw_session`` instead.
    """
    repo = request.registry["cubicweb.repository"]

    if not request.authenticated_userid:
        eid = request.registry.get("cubicweb.anonymous_eid")
        if eid is None:
            return None
        session = repo_connect(request, repo, eid=eid)
    else:
        session = request._cw_cached_session

    return session


def _cw_request(request):
    """Obtains a CubicWeb request wrapper for the pyramid request.

    :param request: A pyramid request
    :return: A CubicWeb request
    :returns type: :class:`CubicWebPyramidRequest`

    Not meant for direct use, use ``request.cw_request`` instead.

    """
    req = CubicWebPyramidRequest(request)
    cnx = request.cw_cnx
    if cnx is not None:
        req.set_cnx(request.cw_cnx)
    return req


def get_principals(login, request):
    """Returns the group names of the authenticated user.

    This function is meant to be used as an authentication policy callback.

    It also pre-open the cubicweb session and put it in
    request._cw_cached_session for later usage by :func:`_cw_session`.

    .. note::

        If the default authentication policy is not used, make sure this
        function gets called by the active authentication policy.

    :param login: A cubicweb user eid
    :param request: A pyramid request
    :returns: A list of group names
    """
    repo = request.registry["cubicweb.repository"]

    try:
        session = repo_connect(request, repo, eid=login)
        request._cw_cached_session = session
    except Exception:
        log.exception("Failed")
        raise

    with session.new_cnx() as cnx:
        with cnx.security_enabled(read=False):
            return {
                group
                for group, in cnx.execute(
                    "Any GN WHERE U in_group G, G name GN, U eid %(userid)s",
                    {"userid": login},
                )
            }


def includeme(config):
    """Enables the core features of Pyramid CubicWeb.

    Automatically called by the 'pyramid' command, or via
    ``config.include('cubicweb.pyramid.code')``. In the later case,
    the following registry entries must be defined first:

    'cubicweb.config'
        A cubicweb 'config' instance.

    'cubicweb.repository'
        The correponding cubicweb repository.

    'cubicweb.registry'
        The vreg.
    """
    repo = config.registry["cubicweb.repository"]

    with repo.internal_cnx() as cnx:
        login = config.registry["cubicweb.config"].anonymous_user()[0]
        if login is not None:
            config.registry["cubicweb.anonymous_eid"] = (
                cnx.find("CWUser", login=login).one().eid
            )

    config.add_request_method(_cw_session, name="cw_session", property=True, reify=True)
    config.add_request_method(_cw_cnx, name="cw_cnx", property=True, reify=True)
    config.add_request_method(_cw_request, name="cw_request", property=True, reify=True)

    cwcfg = config.registry["cubicweb.config"]
    for cube in cwcfg.cubes():
        pkgname = f"cubicweb_{cube}"
        mod = __import__(pkgname)
        if hasattr(mod, "includeme"):
            config.include(pkgname)



from cubicweb_web.request import *  # noqa
from cubicweb_web.request import (  # noqa
    _parse_accept_header,
    _mimetype_sort_key,
    _mimetype_parser,
    _charset_sort_key,
)

from cubicweb.utils import warn_about_deprecated_web_module

warn_about_deprecated_web_module()



# copyright 2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of CubicWeb.
#
# CubicWeb is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# CubicWeb is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with CubicWeb.  If not, see <https://www.gnu.org/licenses/>.
"""Configuration for CubicWeb instances on top of a Pyramid application"""

import random
import string

from logilab.common.configuration import merge_options

from cubicweb.cwconfig import CONFIGURATIONS
from cubicweb.server.serverconfig import ServerConfiguration
from cubicweb_web.webconfig import WebConfiguration


def get_random_secret_key():
    """Return 50-character secret string"""
    chars = string.ascii_letters + string.digits
    secure_random = random.SystemRandom()

    return "".join([secure_random.choice(chars) for i in range(50)])


class AllInOneConfiguration(WebConfiguration, ServerConfiguration):
    """repository and web instance in the same Pyramid process"""

    name = "all-in-one"
    options = merge_options(
        (
            (
                "profile",
                {
                    "type": "string",
                    "default": None,
                    "help": (
                        "profile code and use the specified file to store stats if this option "
                        "is set"
                    ),
                    "group": "web",
                    "level": 3,
                },
            ),
        )
        + WebConfiguration.options
        + ServerConfiguration.options
    )

    cubicweb_appobject_path = (
        WebConfiguration.cubicweb_appobject_path
        | ServerConfiguration.cubicweb_appobject_path
    )
    cube_appobject_path = (
        WebConfiguration.cube_appobject_path | ServerConfiguration.cube_appobject_path
    )


CONFIGURATIONS.append(AllInOneConfiguration)



from os.path import join
from shutil import rmtree

from cubicweb.pyramid.test import PyramidCWTest


class LoginTestLangUrlPrefix(PyramidCWTest):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.config.global_set_option("language-mode", "url-prefix")

    def test_login_password_login_lang_prefix(self):
        res = self.webapp.post(
            "/fr/login", {"__login": self.admlogin, "__password": self.admpassword}
        )
        self.assertEqual(res.status_int, 303)

        res = self.webapp.get("/fr/login")
        self.assertEqual(res.status_int, 303)


class LoginTest(PyramidCWTest):
    def test_login_form(self):
        res = self.webapp.get("/login")
        self.assertIn("__login", res.text)

    def test_login_password_login(self):
        res = self.webapp.post(
            "/login", {"__login": self.admlogin, "__password": self.admpassword}
        )
        self.assertEqual(res.status_int, 303)

        res = self.webapp.get("/login")
        self.assertEqual(res.status_int, 303)

    def test_login_password_login_cookie_expires(self):
        res = self.webapp.post(
            "/login", {"__login": self.admlogin, "__password": self.admpassword}
        )
        self.assertEqual(res.status_int, 303)

        cookies = self.webapp.cookiejar._cookies["localhost.local"]["/"]
        self.assertNotIn("pauth_tkt", cookies)
        self.assertIn("auth_tkt", cookies)
        self.assertIsNone(cookies["auth_tkt"].expires)

        res = self.webapp.logout()
        self.assertEqual(res.status_int, 303)

        self.assertNotIn("auth_tkt", cookies)
        self.assertNotIn("pauth_tkt", cookies)

        res = self.webapp.post(
            "/login",
            {
                "__login": self.admlogin,
                "__password": self.admpassword,
                "__setauthcookie": 1,
            },
        )
        self.assertEqual(res.status_int, 303)

        cookies = self.webapp.cookiejar._cookies["localhost.local"]["/"]
        self.assertNotIn("auth_tkt", cookies)
        self.assertIn("pauth_tkt", cookies)
        self.assertIsNotNone(cookies["pauth_tkt"].expires)

    def test_login_bad_password(self):
        self.config.i18ncompile(["en", "fr"])
        try:
            self.config._gettext_init()
            res = self.webapp.post(
                "/login",
                {"__login": self.admlogin, "__password": "empty"},
                headers={"Accept-Language": "fr"},
                status=403,
            )
        finally:
            rmtree(join(self.config.apphome, "i18n"))
        self.assertIn("\xc9chec de l&#39;authentification", res.text)

    def test_same_site_lax_by_default(self):
        res = self.webapp.post(
            "/login", {"__login": self.admlogin, "__password": self.admpassword}
        )
        for cookie in res.headers.getall("Set-Cookie"):
            self.assertIn("SameSite=Lax", cookie)
        self.assertTrue(len(res.headers.getall("Set-Cookie")) > 0)


class LoginRedirectionTC(PyramidCWTest):
    anonymous_allowed = False

    def test_redirect_to_cubicweb_page_with_arguments(self):
        res = self.webapp.get("/cwsource?args=42", status=303)
        self.assertIn("/login?postlogin_path=/cwsource%3Fargs%3D42", res.text)

    def test_redirect_to_cubicweb_page_without_arguments(self):
        res = self.webapp.get("/cwsource", status=303)
        self.assertIn("/login?postlogin_path=/cwsource", res.text)

    def test_redirect_to_main_page_with_arguments(self):
        res = self.webapp.get("/?args=42", status=303)
        self.assertIn("/login?postlogin_path=/%3Fargs%3D42", res.text)


class CookieParametersTC(PyramidCWTest):
    settings = {
        "cubicweb.auth.authtkt.session.samesite": "None",
    }

    def test_same_site_set_from_config(self):
        res = self.webapp.post(
            "/login", {"__login": self.admlogin, "__password": self.admpassword}
        )
        for cookie in res.headers.getall("Set-Cookie"):
            self.assertIn("SameSite=None", cookie)
        self.assertTrue(len(res.headers.getall("Set-Cookie")) > 0)


if __name__ == "__main__":
    from unittest import main

    main()



from cubicweb_web.view import *  # noqa

from logilab.common.deprecation import (
    TargetMovedDeprecationWarning,
    DeprecationWarningKind,
    send_warning,
)

send_warning(
    'the module "cubicweb.view" is deprecated and has been moved to the cubicweb_web cube, '
    'install it and use "cubicweb_web.view" instead',
    TargetMovedDeprecationWarning,
    deprecation_class_kwargs={
        "kind": DeprecationWarningKind.MODULE,
        "old_name": "view",
        "new_name": "view",
        "old_module": "cubicweb.view",
        "new_module": "cubicweb_web.view",
    },
    stacklevel=3,
    version="3.38",
    module_name="cubicweb",
)



# copyright 2004-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of yams.
#
# yams is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# yams is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with yams. If not, see <https://www.gnu.org/licenses/>.
from datetime import time, date

from yams.buildobjs import EntityType, Datetime, Date, Time
from yams.constraints import TODAY, BoundaryConstraint


class Datetest(EntityType):
    dt1 = Datetime(default="now")
    dt2 = Datetime(default="today")
    d1 = Date(default="today", constraints=[BoundaryConstraint("<=", TODAY())])
    d2 = Date(default=date(2007, 12, 11))
    t1 = Time(default=time(8, 40))
    t2 = Time(default=time(9, 45))
